@@ -10,18 +10,17 @@ testStatistic <- function(fit, designMatrix, critVal, params=getCoefficients(fit
   if(class(fit)[1] == "lme" && class(fit$apVar)[1] == "character") stop(fit$apVar)
   
   ## this is inconsistent theoretically, but consistent with each R model
-  df <- switch(
-               class(fit)[1],
-               lm =, glm = fit$df.residual,
-               gls = fit$dims$N -  fit$dims$p,
-               lme = fit$dims$N -  length(params) - ncol(fit$apVar),      
-               geese = NA)
+  dfdm <- switch(class(fit)[1],
+                 lm =, glm = fit$df.residual,
+                 gls = fit$dims$N -  fit$dims$p,
+                 lme = fit$dims$N -  length(params) - ncol(fit$apVar),      
+                 geese = NA)
   
-  P <- switch(
-              class(fit)[1],
-              lm =, glm = 2 * (1 - pt(abs(testStat), df)),
-              gls = 2 * (1 - pt(abs(testStat), df)),
-              lme = 2 * (1 - pt(abs(testStat), df)),      
+  P <- switch(class(fit)[1],
+              lm =,
+              glm =  if(length(dfdm) > 0 && dfdm[1] > 0) 2 * (1 - pt(abs(testStat), dfdm)) else 2 * (1 - pnorm(abs(testStat), dfdm)),
+              gls = 2 * (1 - pt(abs(testStat), dfdm)),
+              lme = 2 * (1 - pt(abs(testStat), dfdm)),      
               geese = 2 * (1 - pnorm(abs(testStat))))
 
   list(Contrast=est,
@@ -29,7 +28,7 @@ testStatistic <- function(fit, designMatrix, critVal, params=getCoefficients(fit
        Lower=est - critVal * se,
        Upper=est + critVal * se,
        testStat=testStat,
-       df = df,
+       df = dfdm,
        Pvalue=P,
        var=v,
        X=designMatrix)
